unit Unit_TListenerThread;

{$mode ObjFPC}{$H+}

interface

uses
   Classes, SysUtils,
   blcksock,
   synsock,
   unit_const,
   Unit_TTCPThread;

type
{ "Слушающая нить". Ожидает запрос на подключение и управляет потоками
для работы с клиентами }

   TListenerThread = class(TThread)
      private
         FSocket : TTCPBlockSocket; (* объект сокета TSocksBlockSocket *)
         FThreadList : TList; (* список дескрипторов потоков для работы с клиентами *)

      protected
         procedure Execute; override;

      public
         tstr_memo : TStringList;
         num_cycle : integer;
         int64_v1, int64_v2 : int64;
         oldint64_v1 : int64;
         time1, time2 : TDateTime;
         constructor Create(ASyspended: boolean{; const AIP,APort: string});
         destructor Destroy; override;
         property Socket: TTCPBlockSocket read FSocket;
   end;

implementation

constructor TListenerThread.Create(ASyspended: boolean);
begin
   FSocket:=TTCPBlockSocket.Create;
   FThreadList:=TList.Create;
   inherited Create(ASyspended);

   self.tstr_memo:=TStringList.Create;
   self.tstr_memo.Clear;
   self.tstr_memo.Add('TTCPThread.Create: start');
end;


destructor TListenerThread.Destroy;
var
  var_T : TTCPThread;
begin
   (* завершаем все работающие нити *)
   while (FThreadList.Count<>0) do
      begin
         var_T:=TTCPThread(FThreadList.Extract(FThreadList.Last));

         var_T.Terminate;
         var_T.WaitFor;

         var_T.Free;
      end;

   self.tstr_memo.Add('TTCPThread.Destroy: start');
   (* self.tstr_memo.Destroy; *)

   //освобождаем память
   FThreadList.Free;

   FSocket.Free;
   inherited;
end;


procedure TListenerThread.Execute;
var
  var_T : TTCPThread;
begin
   FSocket.CreateSocket;//создаем новый сокет

   //связываем сокет с локальным адресом
   //выбор номера порта оставляем на усмотрение Synapse
   FSocket.Bind(FSocket.LocalName,'502');

   if (FSocket.LastError=0) then //связываение с локальным адресом прошло успешно
      FSocket.Listen //переходим в режим ожидания
   else
      raise Exception.Create(FSocket.LastErrorDesc);//ошибка связывания - показываем её пользователю

   repeat
      if FSocket.CanRead(100) then //можем произвести чтение
         begin
            //получаем дескриптор сокета и создаем новую нить для клиента
            var_T:=TTCPThread.Create(True, FSocket.Accept );

            //определяем обработчик события ONStatus для новой нити
            var_T.Socket.OnStatus:=FSocket.OnStatus;

            //добавляем указатель на нить в список
            FThreadList.Add(pointer(var_T));
            var_T.num_cycle:=self.num_cycle;

            //запускаем нить на выполнение
            var_T.Start;
         end;
   until Terminated;//"гуляем" по циклу до тех пор, пока пользователь не остановит

   self.int64_v1:=var_T.pint64_v1^;
   self.int64_v2:=var_T.pint64_v2^;
   self.oldint64_v1:=var_T.oldint64_v1;
   self.time1:=var_T.time1;
   self.time2:=var_T.time2;

end;

end.

